/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: TopologyException.java, v 1.1 31 aot 07 11:52:47 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.topology;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.objectweb.petals.topology.generated.Topology;
import org.objectweb.petals.topology.joram.JoramWriter;
import org.xml.sax.SAXException;

/**
 * The topology builder allows to unmarshall the topology.xml file.
 *
 * @author Nicolas SALATGE - eBMWebSourcing
 *
 */
public final class TopologyBuilder {

    /**
     * XSD schema
     */
    public static final String NAMESPACE_XSD = "http://www.w3.org/2001/XMLSchema";

    /**
     * Topology schema
     */
    public static final String TOPOLOGY_XSD = "petalsTopology.xsd";

    /**
     * The singleton instance.
     */
    private static TopologyBuilder topologyBuilder;

    /**
     * The unmarshaller.
     */
    private static Unmarshaller unmarshaller;

    /**
     * Default constructor.
     *
     * @throws JAXBException
     *             Impossible to initialize the JAXB parser
     * @throws TopologyException
     *             Impossible to get the JBI schema
     */
    private TopologyBuilder() throws JAXBException, TopologyException {
        final JAXBContext jc = JAXBContext
            .newInstance(new Class[] {Topology.class});
        unmarshaller = jc.createUnmarshaller();
        unmarshaller.setSchema(this.getTopologySchema());
    }

    /**
     * Get the instance of the singleton
     *
     * @return the singleton instance
     * @throws JAXBException
     * @throws TopologyException
     */
    public synchronized static TopologyBuilder getInstance()
        throws JAXBException, TopologyException {
        if (topologyBuilder == null) {
            topologyBuilder = new TopologyBuilder();
        }
        return topologyBuilder;
    }

    /**
     * Create the topology object.
     *
     * @param xmlFileName
     *            the xml file name
     * @return the topology
     * @throws TopologyException
     *             impossible to create the topology
     */
    public Topology createTopology(String xmlFileName) throws TopologyException {
        return createTopology(new File(xmlFileName));
    }

    /**
     * Create the topology object.
     *
     * @param xmlFile
     *            the xml file
     * @return the topology
     * @throws TopologyException
     *             impossible to create the topology
     */
    public Topology createTopology(File xmlFile) throws TopologyException {
        JAXBElement<Topology> root = null;
        try {
            root = unmarshaller.unmarshal(new StreamSource(xmlFile),
                Topology.class);
        } catch (JAXBException e) {
            throw new TopologyException(e);
        }

        return root.getValue();
    }

    /**
     * Write an a3server.xml file for joram
     *
     * @param srcTopology
     *            the topology object
     * @param targetFileNamegetTopologySchema
     *            the targeted file name
     * @throws IOException
     *             impossible to create the file
     * @throws TopologyException
     *             impossible to analyse the topology
     */
    public void writeA3ServerForJoram(final Topology srcTopology,
        final String targetFileName) throws IOException, TopologyException {
        final JoramWriter joram = new JoramWriter(srcTopology);
        joram.writeA3Server(targetFileName);
    }

    /**
     * Get the Topology Schema
     *
     * @return the Topology schema
     * @throws JBIDescriptorException
     *             Impossible to get the Topology schema
     */
    private Schema getTopologySchema() throws TopologyException {
        Schema schema = null;
        final SchemaFactory factory = SchemaFactory.newInstance(NAMESPACE_XSD);

        final Source[] schemas = new StreamSource[1];

        try {
            schemas[0] = new StreamSource(this.openSchemaFile(TOPOLOGY_XSD));

            schema = factory.newSchema(schemas);

        } catch (final IOException e) {
            throw new TopologyException(e);
        } catch (final SAXException e) {
            throw new TopologyException(e);
        }
        return schema;
    }

    /**
     * NOTE: The schema must be available in the ressources of this class'
     * class-loader to be opened.
     *
     * @return JBI schema as InputStream, <code>null</code> if the schema has
     *         not be found in the resources
     * @throws IOException
     *             if an error occurs during InputStream creation from schema
     *             file
     */
    private InputStream openSchemaFile(final String schemaFile)
        throws IOException {
        InputStream theSchemaResource = null;
        // search through classpath obtains from this class classloader
        final URL schemaURL = this.getClass().getResource("/" + schemaFile);

        if (schemaURL != null) {
            theSchemaResource = schemaURL.openStream();
        }

        return theSchemaResource;
    }
}
